@@ -21,7 +21,7 @@ class DeepTile:
         self.image = image
         self.image_type = None
         self.image_shape = None
-        self.link_data = True
+        self.link_data = None
         self.profiles = []
 
     def process(self, tiles, func_process, batch_axis=None, batch_size=None, pad_final_batch=False):
@@ -49,10 +49,14 @@ class DeepTile:
                 Array of tiles after processing with ``func_process``.
         """
 
+        job_kwargs = locals()
+        job_kwargs.pop('self')
+        job_kwargs.pop('tiles')
+
         tiles = utils.to_tuple(tiles)
         self._check_compatibility(tiles, func_process, 'process')
 
-        job = Job(tiles, 'process', locals())
+        job = Job(tiles, 'process', job_kwargs)
 
         profile = job.profile
         nonempty_indices = profile.nonempty_indices
@@ -129,10 +133,14 @@ class DeepTile:
                 Stitched array.
         """
 
+        job_kwargs = locals()
+        job_kwargs.pop('self')
+        job_kwargs.pop('tiles')
+
         tiles = utils.to_tuple(tiles)
         self._check_compatibility(tiles, func_stitch, 'stitch')
 
-        job = Job(tiles, 'stitch', locals())
+        job = Job(tiles, 'stitch', job_kwargs)
 
         stitched = func_stitch(*tiles)
         stitched = Stitched(stitched, job, func_stitch.output_type)
@@ -155,9 +163,13 @@ class DeepTile:
         Raises
         ------
             ValueError
-                If ``tiles`` and ``func`` are not compatible.
+                If ``tiles`` has an input count that does not match the expected ``func`` input count.
             ValueError
-                If ``tiles`` were not created by this DeepTile object.
+                If ``tiles`` has an object type that does not match the expected ``func`` input object type.
+            ValueError
+                If ``tiles`` are not associated with this DeepTile object.
+            ValueError
+                If ``tiles`` do not all share a common profile.
             TypeError
                 If ``func`` has not been transformed to an instance of the Algorithm class.
             TypeError
@@ -171,11 +183,18 @@ class DeepTile:
         if num_expected != num_got:
             raise ValueError(f'Expected input count {num_expected}, got {num_got}.')
 
+        profile = None
         for i, ts in enumerate(tiles):
-            if self is not ts.dt:
-                raise ValueError("Tiles were not created by this DeepTile object.")
             if ts.otype != input_type[i]:
                 raise ValueError(f"Tile object type does not match the expected function input object type.")
+            if profile is None:
+                if ts.dt is self:
+                    profile = ts.profile
+                else:
+                    raise ValueError("Tiles are not associated with this DeepTile object.")
+            else:
+                if ts.profile is not profile:
+                    raise ValueError(f'Tiles must all share a common profile.')
 
         if not issubclass(type(func), AlgorithmBase):
             raise TypeError(f"func_{job_type} must be transformed to an instance of the Algorithm class.")
@@ -218,6 +237,9 @@ class DeepTileArray(DeepTile):
                 Array of tiles.
         """
 
+        job_kwargs = locals()
+        job_kwargs.pop('self')
+
         image = self.image[slices]
         self.image_shape = image.shape
 
@@ -228,7 +250,7 @@ class DeepTileArray(DeepTile):
         nonempty_indices = utils.get_nonempty_indices(tiles)
 
         profile = Profile(self, tiling, tile_size, overlap, slices, nonempty_indices, tile_indices, border_indices)
-        job = Job(self.image, 'get_tiles', locals(), profile)
+        job = Job(self.image, 'get_tiles', job_kwargs, profile)
         tiles = Tiled(tiles, job, 'tiled_image')
 
         return tiles
@@ -268,6 +290,9 @@ class DeepTileLargeImage(DeepTile):
                 Array of tiles.
         """
 
+        job_kwargs = locals()
+        job_kwargs.pop('self')
+
         from deeptile.sources import large_image
 
         self.image_shape = (self.image.getMetadata()['sizeY'], self.image.getMetadata()['sizeX'])
@@ -277,7 +302,7 @@ class DeepTileLargeImage(DeepTile):
         nonempty_indices = utils.get_nonempty_indices(tiles)
 
         profile = Profile(self, tiling, tile_size, overlap, slices, nonempty_indices, tile_indices, border_indices)
-        job = Job(self.image, 'get_tiles', locals(), profile)
+        job = Job(self.image, 'get_tiles', job_kwargs, profile)
         tiles = Tiled(tiles, job, 'tiled_image')
 
         return tiles
@@ -317,6 +342,9 @@ class DeepTileND2(DeepTile):
                 Array of tiles.
         """
 
+        job_kwargs = locals()
+        job_kwargs.pop('self')
+
         from deeptile.sources import nd2
 
         tiles, tiling, tile_size, overlap, self.image_shape = nd2.parse(self.image, self.image_sizes, self.axes_order,
@@ -325,7 +353,7 @@ class DeepTileND2(DeepTile):
         nonempty_indices = utils.get_nonempty_indices(tiles)
 
         profile = Profile(self, tiling, tile_size, overlap, slices, nonempty_indices, tile_indices, border_indices)
-        job = Job(self.image, 'get_tiles', locals(), profile)
+        job = Job(self.image, 'get_tiles', job_kwargs, profile)
         tiles = Tiled(tiles, job, 'tiled_image')
 
         return tiles
